# Clean 5% outliers in distribution (take 95% of the distribution)
remove_outliers <- function(x, na.rm=TRUE, ...) {
  qnt <- quantile(x, probs=c(.025, .975), na.rm=na.rm, ...)
  H <- 1.5 * IQR(x, na.rm=na.rm)
  y <- x
  y[x < (qnt[1] - H)] <- NA
  y[x > (qnt[2] + H)] <- NA
  y
}

start_time <- Sys.time()
# Create 10000 noisy networks with 5% error from the musculoskeletal matrices
set.seed(73)
simulite <- 10000 # sample size of noisy networks
noisy_networks_skeletal <- list()
for (i in 1:length(networks)){ # for each network
  g <- graph.list[[i]]
  noisy_gs <- list()
  for (j in 1:simulite){
    # multiple links and loops are allowed in the rewiring, but are then removed by simplifying the network
    # this allows to produce variation of links (variation in K and D)
    ngs <- simplify(rewire(g, each_edge(p=.05, loops=T, multiple=T))) # 5% variation
    cl <- igraph::components(ngs) # to make sure we take the largest connected component of the network (this allows variation in N)
    noisy_gs[[j]] <- induced_subgraph(ngs, vids=which(cl$membership == which.max(cl$csize)))
  }
  noisy_networks_skeletal[[i]] <- noisy_gs
}

# Calculate parameters for noisy skeletal networks (to test robustness)
mean_parameters <- sd_parameters <- parameters # mean and sd of the noisy sample
noisy_parameters <- rep(list(list()), length(networks)) 
for (i in 1:length(networks)){
  noisy_parameters[[i]][[1]] <- unlist(mapply(vcount, noisy_networks_skeletal[[i]])) # N
  noisy_parameters[[i]][[1]] <- remove_outliers(noisy_parameters[[i]][[1]])
  mean_parameters[i,1] <- mean(noisy_parameters[[i]][[1]], na.rm=TRUE)
  sd_parameters[i,1] <- sd(noisy_parameters[[i]][[1]], na.rm=TRUE)
  noisy_parameters[[i]][[2]] <- unlist(mapply(ecount, noisy_networks_skeletal[[i]])) # K
  noisy_parameters[[i]][[2]] <- remove_outliers(noisy_parameters[[i]][[2]])
  mean_parameters[i,2] <- mean(noisy_parameters[[i]][[2]], na.rm=TRUE)
  sd_parameters[i,2] <- sd(noisy_parameters[[i]][[2]], na.rm=TRUE)
  noisy_parameters[[i]][[3]] <- unlist(mapply(edge_density, noisy_networks_skeletal[[i]])) # D
  noisy_parameters[[i]][[3]] <- remove_outliers(noisy_parameters[[i]][[3]])
  mean_parameters[i,3] <- mean(noisy_parameters[[i]][[3]], na.rm=TRUE)
  sd_parameters[i,3] <- sd(noisy_parameters[[i]][[3]], na.rm=TRUE)
  noisy_parameters[[i]][[4]] <- unlist(mapply(transitivity, noisy_networks_skeletal[[i]], type="average", isolates="zero")) # C
  noisy_parameters[[i]][[4]] <- remove_outliers(noisy_parameters[[i]][[4]])
  mean_parameters[i,4] <- mean(noisy_parameters[[i]][[4]], na.rm=TRUE)
  sd_parameters[i,4] <- sd(noisy_parameters[[i]][[4]], na.rm=TRUE)
  noisy_parameters[[i]][[5]] <- unlist(mapply(mean_distance, noisy_networks_skeletal[[i]], directed=FALSE)) # L
  noisy_parameters[[i]][[5]] <- remove_outliers(noisy_parameters[[i]][[5]])
  mean_parameters[i,5] <- mean(noisy_parameters[[i]][[5]], na.rm=TRUE)
  sd_parameters[i,5] <- sd(noisy_parameters[[i]][[5]], na.rm=TRUE)
  noisy_parameters[[i]][[6]] <- unlist(mapply(heterogeneity, noisy_networks_skeletal[[i]])) # H
  noisy_parameters[[i]][[6]] <- remove_outliers(noisy_parameters[[i]][[6]])
  mean_parameters[i,6] <- mean(noisy_parameters[[i]][[6]], na.rm=TRUE)
  sd_parameters[i,6] <- sd(noisy_parameters[[i]][[6]], na.rm=TRUE)
  noisy_parameters[[i]][[7]] <- unlist(mapply(assortativity_degree, noisy_networks_skeletal[[i]], directed=FALSE)) # A
  noisy_parameters[[i]][[7]] <- remove_outliers(noisy_parameters[[i]][[7]])
  mean_parameters[i,7] <- mean(noisy_parameters[[i]][[7]], na.rm=TRUE)
  sd_parameters[i,7] <- sd(noisy_parameters[[i]][[7]], na.rm=TRUE)
  n_parcellation <- n_parcellation_clean <- vector() # P
  for (j in 1:length(noisy_networks_skeletal[[i]])){
    n_membership <- cluster_fast_greedy(noisy_networks_skeletal[[i]][[j]])$membership # for P we calculate one partition for each noisy network
    n_parcellation[j] <- parcellation_index(n_membership)
  }
  noisy_parameters[[i]][[8]] <- n_parcellation
  mean_parameters[i,8] <- mean(noisy_parameters[[i]][[8]], na.rm=TRUE)
  sd_parameters[i,8] <- sd(noisy_parameters[[i]][[8]], na.rm=TRUE)
}
noisy_parameters_skeletal <- noisy_parameters
mean_noisy_parameters_skeletal <- mean_parameters
sd_noisy_parameters_skeletal <- sd_parameters
sem_noisy_parameters_skeletal <- sd_noisy_parameters_skeletal / sqrt(length(noisy_networks_skeletal[[1]])) # standard error of the mean

# Save gross result
write.xlsx(mean_noisy_parameters_skeletal, file="Extra Supplementary Table. Mean values of parameters in 10000 noisy skeletal networks.xlsx", sheetName="Sheet1", row.names=TRUE) # save externally
write.xlsx(sd_noisy_parameters_skeletal, file="Extra Supplementary Table. SD values of parameters in 10000 noisy skeletal networks.xlsx", sheetName="Sheet1", row.names=TRUE)
write.xlsx(sem_noisy_parameters_skeletal, file="Extra Supplementary Table. SEM values of parameters in 10000 noisy skeletal networks.xlsx", sheetName="Sheet1", row.names=TRUE)

# Is observation within the range of the sample of noisy networks?
# H0: Observation=mean of the sample
# Ha: Observation ≠ mean of the sample
# alpha = 0.05
lower_Q <- matrix(FALSE, nrow=length(networks), ncol=8)
rownames(lower_Q) <- networks
colnames(lower_Q) <- parameter_names
upper_Q <- lower_Q
within_IQR <- lower_Q
for (i in 1:length(networks)){
  for (j in 1:8){
    lower_Q[i,j] <- quantile(noisy_parameters_skeletal[[i]][[j]], probs=c(0, 0.025, 0.5, 0.975, 1), type=8, names=FALSE, na.rm=TRUE)[2]
    upper_Q[i,j] <- quantile(noisy_parameters_skeletal[[i]][[j]], probs=c(0, 0.025, 0.5, 0.975, 1), type=8, names=FALSE, na.rm=TRUE)[4]
    if (parameters[i,j] >= lower_Q[i,j] & parameters[i,j] <= upper_Q[i,j]){
      within_IQR[i,j] <- TRUE
    }
  }
}
m <- cbind(lower_Q, upper_Q) # combine
Q95_skeletal <- m[, c(matrix(1:ncol(m), nrow=2, byrow=T))] # then reorder
colnames(Q95_skeletal) <- c("Lower N", "Upper N", "Lower K", "Upper K", "Lower D", "Upper D", "Lower C", "Upper C", "Lower L", "Upper L", "Lower H", "Upper H", "Lower A", "Upper A", "Lower P", "Upper P")
within_IQR_skeletal <- within_IQR

# Values of TRUE mean that we cannot reject H0. Thus, the observed value falls within the range of the average noisy networks.
write.xlsx(Q95_skeletal, file="Extra Supplementary Table. Inter 95% distribution range values of parameters in 10000 noisy skeletal networks.xlsx", sheetName="Sheet1", row.names=TRUE)
write.xlsx(within_IQR_skeletal, file="Supplementary Table 00. Test of observed values in skeletal network falling within the 95% range of 10000 noisy skeletal networks.xlsx", sheetName="Sheet1", row.names=TRUE)

end_time <- Sys.time()
print(end_time - start_time) # checks how long it takes